/* 
 * File:   main.cpp
 * Author: Mohsen Reisi
 *
 * Created on 31 May 2015, 10:03 AM
 */

/* 
This program reads the counts of the specific 9-tuples and determines all 
statistical variables which are presented in Ganz's paper.
*/
#include <iostream>
#include <vector>
#include <fstream>
#include <string>
#include <algorithm>
#include <cmath>

using namespace std;

int getNBlocks()
{
    ifstream countsFile("counts.txt");
    string line;
    int nLines = 0;
    while (getline(countsFile, line))
        ++nLines;
    countsFile.close();
    cout << "number of blocks= " << nLines << endl;
    return (nLines);
}

void getX(int a, int nBlocks)
{
    
    ifstream countsFile("counts.txt");
    ofstream XFile("X_i.txt");

    for (int i = 0; i < nBlocks; ++i)
    {
        int temp;
        countsFile >> temp;
        if (temp <= a)
            XFile << 0;
        else
            XFile << 1;
        if (i != nBlocks - 1)
            XFile << endl;
    }
    countsFile.close();
    XFile.close();
    cout << "X_i have been determined!" << endl;
}

void getL(int nBlocks)
{
    ifstream XFile("X_i.txt");
    ofstream LFile("L_j.txt");
    vector<int> X;

    for (int i = 0; i < nBlocks; ++i)
    {
        int temp;
        XFile >> temp;
        X.push_back(temp);
    }
    XFile.close();

    int prev = X[0];
    int n = 1;
    for (int i = 1; i < nBlocks; ++i)
    {
        if (X[i] == prev)
            ++n;
        else
        {
            LFile << n << endl;
            n = 1;
            prev = X[i];
        }
    }
    LFile << n;
    LFile.close();
    cout << "L_j have been determined!" << endl;
}

int getD(int nBlocks, vector<int>& d, vector<int>& D)
{
    ifstream LFile("L_j.txt");
    ofstream dFile("d_k.txt");
    ofstream DFile("DD_k.txt");

    vector<int> L;

    string line;
    while (getline(LFile, line))
        L.push_back(atoi(line.c_str()));
    LFile.close();

    int r = L.size();
    int n = floor(r / 2);

    int k=0;
    for (int j = 0; j < r - 1; j+=2)
    {
        d.push_back(L[j] - L[j + 1]);
        dFile << d[k++];
        if (j != r - 2)
            dFile << endl;
    }
    cout << "d_k have been determined! r= " << r << endl;

    for (int j = 0; j < n; ++j)
    {
        D.push_back(L[j] - L[j + n]);
        DFile << D[j];
        if (j != n - 1)
            DFile << endl;
    }

    cout << "D_k have been determined! n= " << n << endl;
    dFile.close();
    DFile.close();
	return(n);
}

void getTeta(vector<int>& dD, int n)
{
    int T = 0;
	//vector<vector<vector<int> > > Z (n,vector<vector<int> >(n,vector <int>(n)));
    vector< vector< vector<int> > > Z;
    Z.resize(n);
    for (int k = 0; k < n; ++k)
    {
        Z[k].resize(n);
        for (int l = 0; l < n; ++l)
            Z[k][l].resize(n);
    }

    for (int k = 0; k < n - 2; ++k)
    {
        for (int l = k + 1; l < n - 1; ++l)
            for (int m = l + 1; m < n; ++m)
            {
                int Z_klm = 0;
                if (dD[k] - 2 * dD[l] + dD[m] > 0)
                    Z_klm = 1;
                else if (dD[k] - 2 * dD[l] + dD[m] < 0)
                    Z_klm = -1;
                Z[k][l][m] = Z_klm;
                T += Z_klm;
            }
    }
    cout << "T= " << T << endl;

   vector<int> T_r;
    for (int r = 0; r < n; ++r)
    {
        int temp = 0;
        for (int k = 0; k < n - 2; ++k)
            for (int l = k + 1; l < n - 1; ++l)
                for (int m = l + 1; m < n; ++m)
                    if ((k == r) || (l == r) || (m == r))
                        temp += Z[k][l][m];
        T_r.push_back(temp);
    }

    vector< vector<int> > T_st;
    T_st.resize(n);
    for (int s = 0; s < n; ++s)
        T_st[s].resize(n);

    for (int s = 0; s < n - 1; ++s)
        for (int t = s + 1; t < n; ++t)
        {
            int temp = 0;
            for (int k = 0; k < n - 2; ++k)
                for (int l = k + 1; l < n - 1; ++l)
                    for (int m = l + 1; m < n; ++m)
                        if (((k == s) || (l == s) || (m == s))&&
                                ((k == t) || (l == t) || (m == t)))
                            temp += Z[k][l][m];
            T_st[s][t]=temp;
        }

    double V = 0;
    for (int r = 0; r < n; ++r)
        V += ((double) ((n - 3)*(n - 4)) / ((n - 1)*(n - 2))) * T_r[r] * T_r[r];
    for (int s = 0; s < n - 1; ++s)
        for (int t = s + 1; t < n; ++t)
            V += ((double) (n - 3) / (n - 4)) * T_st[s][t] * T_st[s][t];
    V += (double) (n * (n - 1)*(n - 2)) / 6;
    V -= (1 - (double) ((n - 3)*(n - 4)*(n - 5)) / (n * (n - 1)*(n - 2))) * T*T;
    cout << "V= " << V << endl;
    double teta = (double) T / (sqrt(V));
    cout << "teta= " << teta << endl;
    
    Z.clear();
    T_r.clear();
	T_st.clear(); 
}
/*The input is the expected value for the number of 9-tuples with at least five consecutive identical digits like 124666663
Example: command ./statisticalTest 2300000 works where the number of blocks is 222 (there are 4.50E+10 digit in each block). 
*/
int main(int argc, char * argv[])
{
    int a = atoi(argv[1]);
    int nBlocks = getNBlocks();
    vector<int> d, D;
    getX(a,nBlocks);
    getL(nBlocks);
    int n = getD(nBlocks, d, D);
    
    sort(d.begin(), d.end());
    getTeta(d, n);
    
    sort(D.begin(), D.end());
    getTeta(D, n);
    return 0;
}
